<?php
/*
Plugin Name: FriendFeed Comments
Version: 1.2
Plugin URI: http://blog.slaven.net.au/wordpress-plugins/friendfeed-comments-wordpress-plugin/
Description: Places the comments from friend feed on each post from your blog under the normal comments section on the post page.
Author: Glenn Slaven
Author URI: http://blog.slaven.net.au/

	Copyright 2008 Glenn Slaven  (email : gdalziel@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
require_once("friendfeed.php");

class FriendFeedPostComments
{
	var $options_key = 'friendfeed_options';
	var $discussions_key = 'friendfeed_discussions';
	var $max_posts_to_load = 150;
	var $page_size = 30;
	var $is_debug = false;
	
	function FriendFeedPostComments()
	{
		add_action('admin_menu', array(&$this, '_create_options_menu_item'));
		add_action('update_friendfeedcomments_hook', array(&$this, 'update_friendfeedcomments'));
		
		if (method_exists($this, '_install')) { register_activation_hook(__FILE__, array(&$this, '_install')); }
		if (method_exists($this, '_uninstall')) { register_deactivation_hook(__FILE__, array(&$this, '_uninstall')); }
	}
	
	//Callback to create the Settings menu option
	function _create_options_menu_item(){
		if (function_exists('add_options_page')) { add_options_page("FriendFeed", "FriendFeed", 9, __FILE__, array(&$this, 'options_page')); }
		add_action('admin_notices', array(&$this, '_admin_notices'));
	}
	
	//Display admin notices if needed
	function _admin_notices() {
		$options = get_option($this->options_key);
		if ((!$options || !$options['nickname']) && !$_POST['ff_nickname'])
		{
			echo "<div class='updated' style='background-color:#f66;'><p><a href=\"options-general.php?page=friendfeed-comments/friendfeed-comments.php\">FriendFeed Comments</a> does not have a nickname set.  You need to enter your FriendFeed nickname for the comments to be loaded.</p></div>";
		}
	}
	
	//Check if the debug flag is set
	function is_debug() {
		if ($_REQUEST['ffdebugflag']) {
			$this->is_debug = true;
		}
		
		return $this->is_debug;
	}
	
	//Callback to setup the plugin
	function _install() {
		wp_schedule_event( time(), 'hourly', 'update_friendfeedcomments_hook' );
		$options = get_option($this->options_key);
		if (!$options) {
			$options['nickname'] = '';
			$options['servicename'] = 'blog';
			update_option($this->options_key, $options);
		}
		
	}
	
	//Callback to unstall the plugin
	function _uninstall() {
		wp_clear_scheduled_hook('update_friendfeedcomments_hook');
	}
	
	//Format the title into a normalised string that can be used as a hash key
	function format_title($title) {
		return md5(urlencode(ereg_replace("[/,\s\:\"]", "", strtolower(str_replace(urldecode('&#8217;'), '', $title)))));
	}
	
	//Debug output
	function printr($string) {
		print "<pre>";
		print_r($string);
		print "</pre>";
	}
	
	//Callback to update the comments from the FriendFeed API
	function update_friendfeedcomments() {
		$friendfeed = new FriendFeed('dalziel');
		$options = get_option($this->options_key);
		$ffposts = $friendfeed->fetch_user_feed($options['nickname'], $options['servicename']);
		
		if ($ffposts)
		{
			//Load up the posts from FriendFeed up to the max number of posts to load set
			for ($i = $this->page_size ; $i <= $this->max_posts_to_load - $this->page_size; $i = $i + $this->page_size) {
				$temp_posts = $friendfeed->fetch_user_feed($options['nickname'], $options['servicename'], $i, $this->page_size);
				
				if ($temp_posts && is_array($temp_posts->entries)) {
					$ffposts->entries = array_merge($ffposts->entries, $temp_posts->entries);
				} else {
					$i = $this->max_posts_to_load; 
				}
			}
			
			if ($this->is_debug()) {
				$this->printr(count($ffposts->entries) . " entries loaded from FriendFeed");
			}
		
			$ffdiscussions = get_option($this->discussions_key);
			$blog_url = parse_url(get_option('siteurl'));
			
			$match_count = 0;
			foreach($ffposts->entries as $post)
			{
				$post_url = parse_url($post->service->profileUrl);
				if (($post->comments || $post->likes) && $blog_url['host'] == $post_url['host'] && $blog_url['path'] == $post_url['path'])
				{
					$title = $this->format_title($post->title);
					$ffdiscussions[$title]['id'] = $post->id;
					$ffdiscussions[$title]['link'] = $post->link;
					$ffdiscussions[$title]['comments'] = $post->comments;
					$ffdiscussions[$title]['likes'] = $post->likes;
					$match_count++;
				}
			}	
			update_option($this->discussions_key, $ffdiscussions);
		
			if ($this->is_debug()) {
				$this->printr("$match_count entries loaded, " . count($ffdiscussions) . " total entries stored.");
				$this->printr($ffdiscussions);
				$this->printr($ffposts);
			}

		}	
	}
	
	//Function to render the options page
	function options_page() {
		$options = get_option($this->options_key);
		if ($_POST["wp_friendfeed"])
		{
			if ($_POST['ff_showdebug']) {
				$this->is_debug = true;
			}
		
			if ($_POST['Clear']) {
				delete_option($this->discussions_key);
				$this->update_friendfeedcomments();
				$message = "<strong>Discussions reloaded.</strong>";
			} else {
				$options['nickname'] = $_POST['ff_nickname'];
				$options['servicename'] = $_POST['ff_servicename'];
				update_option($this->options_key, $options);
				$this->update_friendfeedcomments();
				$message = "<strong>Settings Saved.</strong>  Don't forget to put <code>&lt;?php wp_ffcomments(); ?&gt;</code> on your single post template to display the FriendFeed comments &amp; likes.";
			}
?>
	<div id="message" class="updated fade" style="background-color: rgb(255, 251, 204);"><p><?=$message?></p></div>
<?php
		}
?>
	<div class="wrap">
		<form method="post">
			<input type="hidden" value="true" name="wp_friendfeed"/>
			<h2>FriendFeed Comments</h2>
			<table class="form-table">
			<tbody>
				<tr valign="top">
					<th scope="row">FriendFeed Nickname</th>
					<td><input type="text" size="40" value="<?=$options['nickname']?>" id="ff_nickname" name="ff_nickname" /></td>
				</tr>
				<tr valign="top">
					<th scope="row">FriendFeed Servicename</th>
					<td>
						<input type="text" size="40" value="<?=$options['servicename']?>" id="ff_servicename" name="ff_servicename" />
						<br />This is the name that FriendFeed knows your blog as.  Unless you don't use the 'blog' service on FriendFeed, you probably won't need to change this.
					</td>
				</tr>
				<tr valign="top">
					<th scope="row">Display debug information</th>
					<td>
						<input type="checkbox" name="ff_showdebug" id="ff_showdebug" value="1" />
						<br />Check this if you want to see the information that is being loaded by the plugin when you save.  Only needed for troubleshooting.
					</td>
				</tr>
			</tbody>
			</table>
			<p class="submit"><input type="submit" value="Save Changes" name="Submit"/> <input type="submit" style="color:#DD0000; font-weight:bold;" value="Reset comments/likes" name="Clear" onclick="return confirm('This will delete, then reload all FriendFeed comments & likes.  Are you sure?');" /></p>
		</form>
		<div style="background-color:rgb(238, 238, 238); border: 1px solid rgb(85, 85, 85); padding: 5px; margin-top:10px;">
			<p>Did you find this plugin useful?  Please consider donating to help me continue developing it and other plugins.</p>
			<form action="https://www.paypal.com/cgi-bin/webscr" method="post"><input type="hidden" name="cmd" value="_xclick" /><input type="hidden" name="business" value="paypal@slaven.net.au" /><input type="hidden" name="item_name" value="FriendFeed Comments Wordpress Plugin" /><input type="hidden" name="no_note" value="1" /><input type="hidden" name="currency_code" value="AUD" /><input type="hidden" name="tax" value="0" /><input type="hidden" name="bn" value="PP-DonationsBF" /><input type="image" src="https://www.paypal.com/en_US/i/btn/x-click-but04.gif" border="0" name="submit" alt="Make payments with PayPal - it's fast, free and secure!" /></form>
		</div>
	</div>
<?php
	}
	
	function display_comments() {
		$ffdiscussions = get_option($this->discussions_key);
		$title = $this->format_title(the_title('','',false));
		
		if ($this->is_debug()) {
			$this->printr($ffdiscussions);
			$this->printr($title);
		}
		
		if ($ffdiscussions && $ffdiscussions[$title]) {
			$timeformat = get_option('time_format');
			$dateformat = get_option('date_format');
			$discussion = $ffdiscussions[$title];

			if ($discussion['comments'] || $discussion['likes']) {		
?>
	<script type="text/javascript">
	function toggleFriendFeed(link)
	{
		fflist = document.getElementById('friendfeeddiscussions');
		
		if (fflist && link)
		{
			if (fflist.style.display == 'none')
			{
				fflist.style.display = 'block';
				link.innerHTML = 'hide';
			}
			else
			{
				fflist.style.display = 'none';
				link.innerHTML = 'show';			
			}
		}
	}
	</script>
<?php				
				print "\t" . '<div id="friendfeedcomments">' . "\n";
				print "\t" . '<div id="friendfeedcommentslink">';
				print '<img src="/' . PLUGINDIR . '/friendfeed-comments/friendfeedicon.png" alt="" id="friendfeedicon" />On FriendFeed, this post was ';
				if ($discussion['likes']) {
					$count = count($discussion['likes']);
					print "liked by " . $count . " " . ($count <> 1 ? 'people' : 'person');
					
					if ($discussion['comments']) {
						print " and ";
					}
				}
				
				if ($discussion['comments']) {
					$count = count($discussion['comments']);
					print " commented on " . $count . ($count <> 1 ? ' times' : ' time');
					
				}
				
				print " <a href=\"#\" id=\"togglefriendfeedcommentslink\" onclick=\"toggleFriendFeed(this); return false;\" onfocus=\"blur();\">show</a>";
				
				print "</div>\n";
				print "\t<div id=\"friendfeeddiscussions\" style=\"display:none;\">\n";
				print "\t\t<div class=\"friendfeedpermalink\"><a href=\"http://www.friendfeed.com/e/{$discussion['id']}\">View this post on FriendFeed</a></div>\n";
				if ($discussion['likes']) {
					print "\t\t<div id=\"friendfeedlikeslist\">Liked by\n\t\t\t<ul>\n";
					$count = count($discussion['likes']);
					for($i = 0; $i < $count; $i++) {
						$like = $discussion['likes'][$i];
						print "\t\t\t\t<li><a href=\"{$like->user->profileUrl}\">{$like->user->name}</a>" . ($i < $count - 1 ? ", " : '') . "</li>\n";
					}
					print "\t\t\t</ul>\n\t\t</div>\n";
				}
				
				if ($discussion['comments'])
				{
					print "\t\t<div id=\"friendfeedcommentslist\">\n\t\t<ul>\n";
					foreach($discussion['comments'] as $comment)
					{
						$time = date($timeformat, $comment->date);
						$date = date($dateformat, $comment->date);
						print "\t\t\t<li class=\"friendfeedcomment\">\n";
						print "\t\t\t\t<a href=\"{$comment->user->profileUrl}\"><img style=\"float:left; margin: 0 5px 5px 0px;\" src=\"http://friendfeed.com/{$comment->user->nickname}/picture?size=medium\" alt=\"\" /></a>";
						print "\t\t\t\t<div class=\"friendfeedmeta\"><em>{$date} at {$time}</em> <cite><a href=\"{$comment->user->profileUrl}\">{$comment->user->name}</a></cite></div>\n";
						print "\t\t\t\t<div class=\"friendfeedcommenttext\">" . make_clickable($comment->body) . "</div><div style=\"clear:both;\"></div>\n";
						print"\t\t\t</li>\n";
					}
					print "\t\t</ul>\n\t\t</div>\n";
				}
				print "\t</div>\n\t</div>\n";
			}
		}
	}
}

$FriendFeedPostComments = new FriendFeedPostComments();

function wp_ffcomments()
{
	global $FriendFeedPostComments;
	$FriendFeedPostComments->display_comments();
}
?>